/*
 * Created on Feb 18, 2005
 *
 * 
 */
//package yaes.application.eel6938.socialpotentialfields_svanderweide_agents;
package yaes.application.eel6938.classagents;

import java.awt.Point;

import yaes.application.eel6938.game.objects.Agent;
import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.game.Configuration;

/**
 * @author vanderweides
 *
 * An agent for the Feed, Fight, Mate Simulation which implements the social potential fields framework.
 * There are three forces influencing the Agent, other agents, obstacles, and food. Obstacles cause strictly
 * repulsive forces. Food causes strictly attractive forces. Agents cause attractive forces at long distances
 * and repulsive forces at short distances. An Agent will maintain it's previous direction and a speed when no
 * forces are acting upon it, i.e. it conserves momentum. If an Agent gets within half the critical distance
 * this Agent will attack it. This was based on the fact that as an agent population increases the Agents tend
 * to get closer than the critical distance towards each other. At a certain point, aritrarily chosen, the
 * population is considered overpopulated and should compensate. If this Agent aquires more than 1.5 times the
 * mating energy cost, it mates. When food is abundant, energy reaches the previously stated threshold, the
 * environment is deemed capable of supporting more Agents.
 * 
 * There has been some additional logic added to compensate for the local minima problem. This includes an
 * added "drive" towards food if within a given distance. Also, the agent is not allowed to do a 180 degree
 * turn. This helps prevent it from bouncing back and forth between two obstacles indefinately.
 */
public class PotentialFieldsAgentVer2c extends Agent
{
	public static int creationNumber = 0;
	
  public static final double c1Agent = 45.0;
  public static final double c2Agent = 20.0;
  public static final double sig1Agent = 1.0;
  public static final double sig2Agent = 0.7;

  public static final double c1Food = 0;
  public static final double c2Food = 20.0;
  public static final double sig1Food = 1.0;
  public static final double sig2Food = 0.8;

  public static final double c1Obstacle = 5.0;
  public static final double c2Obstacle = 0.0;
  public static final double sig1Obstacle = 5.0;
  public static final double sig2Obstacle = 1.0;
  
  private int previousMoveDirection;
  
  private boolean shouldAttack = false;
  private Point attackPosition;

  private int[] myColor = new int[3];
  private String myAgentName = "SocialPotentialFields_SVanderWeide";
	
	private int moveDirection = Constants.DirectionEast;
	private int moveSpeed = 0;

	private boolean printDebug = false;
	
  /**
   * 
   */
  public PotentialFieldsAgentVer2c()
  {
    super();
    
    previousMoveDirection = -1;
    myColor[0] = 0;
    myColor[1] = 255;
    myColor[2] = 127;

		creationNumber++;
		
//		if(creationNumber == 1)
//			printDebug = true;
	}

  /* (non-Javadoc)
   * @see yaes.application.eel6938.game.objects.Agent#intelligence()
   */
  public void intelligence()
  {
    Point [] agentPositions = getSensorAgentList();
    Point [] foodPositions = getSensorFoodList();
    Point [] obstaclePositions = getSensorObstacleList();

    double agentsForceMagnitude = 0.0;
    double agentSumX = 0.0;
    double agentSumY = 0.0;
		
    double foodForceMagnitude = 0.0;
    double foodSumX = 0.0;
    double foodSumY = 0.0;
    
    double obstacleForceMagnitude = 0.0;
    double obstacleSumX = 0.0;
    double obstacleSumY = 0.0;

	  double forceX = 0.0;
	  double forceY = 0.0;
		
    double criticalDistance = Math.pow((c1Agent / c2Agent), 1.0 / (sig1Agent - sig2Agent));

		if(printDebug) System.out.println("********************************************************************************");

    // Sum the forces exerted by the agents within sensor range.
    for(int i = 1; i < agentPositions.length; i++)
    {
      Point position = agentPositions[i];
      double distance = Math.sqrt(position.getX() * position.getX() +
                                  position.getY() * position.getY());

      // Attack if an agent gets too close.
      if((distance < criticalDistance / 2.0) && canAttack())
      {
        shouldAttack = true;
        attackPosition = position;
        break;
      }
      else if(getName(position).equals(myAgentName))
			{
        agentsForceMagnitude += -c1Agent / Math.pow(distance, sig1Agent) +
                               c2Agent / Math.pow(distance, sig2Agent);

        agentSumX += position.getX() / distance;
        agentSumY += position.getY() / distance;
			}
    }

		// Sum the forces exerted by the foods within sensor range.
    for(int i = 0; i < foodPositions.length; i++)
    {
      Point position = foodPositions[i];
      double distance = Math.sqrt(position.getX() * position.getX() +
                                  position.getY() * position.getY());

      foodForceMagnitude += -c1Food / Math.pow(distance, sig1Food) +
                             c2Food / Math.pow(distance, sig2Food);

      foodSumX += position.getX() / distance;
      foodSumY += position.getY() / distance;
    }

    // Sum the forces exerted by the obstacles within sensor range.
    for(int i = 0; i < obstaclePositions.length; i++)
    {
      Point position = obstaclePositions[i];
      double distance = Math.sqrt(position.getX() * position.getX() +
                                  position.getY() * position.getY());

      obstacleForceMagnitude += -c1Obstacle / Math.pow(distance, sig1Obstacle) +
                                c2Obstacle / Math.pow(distance, sig2Obstacle);

      obstacleSumX += position.getX() / distance;
      obstacleSumY += position.getY() / distance;
    }

		if(printDebug) System.out.println("agentsForceMagnitude = " + agentsForceMagnitude);
		if(printDebug) System.out.println("foodForceMagnitude = " + foodForceMagnitude);
		if(printDebug) System.out.println("obstacleForceMagnitude = " + obstacleForceMagnitude);

		forceX = agentsForceMagnitude * agentSumX + foodForceMagnitude * foodSumX + obstacleForceMagnitude * obstacleSumX;

    forceY = agentsForceMagnitude * agentSumY + foodForceMagnitude * foodSumY + obstacleForceMagnitude * obstacleSumY;

		if(printDebug) System.out.println("forceX = " + forceX);
		if(printDebug) System.out.println("forceY = " + forceY);

		if(forceX != 0.0 || forceY != 0.0)
    {
      moveDirection = getMyDirection(forceX, forceY);
			
			double forceMagnitude = Math.sqrt(forceX * forceX + forceY * forceY);
			
	    moveSpeed = (int)(4 * forceMagnitude / Configuration.AgentInitialSensorRange) + 1;
			if(moveSpeed > 4)
	      moveSpeed = 4;
    }
    else
			moveSpeed = 1;

		if(printDebug) System.out.println("moveDirection = " + Constants.Directions[moveDirection]);
		if(printDebug) System.out.println("moveSpeed = " + moveSpeed);

    if(foodPositions.length > 0) // Let's avoid overshooting our food or oscillating around it.
    {
      double closestFoodX = foodPositions[0].getX();
      double closestFoodY = foodPositions[0].getY();
      double closestFoodDist = Math.sqrt(closestFoodX * closestFoodX + closestFoodY * closestFoodY);
      
      if((moveDirection == Constants.DirectionNorth && previousMoveDirection == Constants.DirectionSouth) ||
         (moveDirection == Constants.DirectionSouth && previousMoveDirection == Constants.DirectionNorth) ||
         (moveDirection == Constants.DirectionEast && previousMoveDirection == Constants.DirectionWest) ||
         (moveDirection == Constants.DirectionWest && previousMoveDirection == Constants.DirectionEast) ||
         (closestFoodDist < 4.0))
      {
				if(printDebug) System.out.println("FOOD CHANGED");

				if(printDebug) System.out.println("closestFoodX = " + closestFoodX);
				if(printDebug) System.out.println("closestFoodY = " + closestFoodY);

	      moveDirection = getMyDirection(foodPositions[0].getX(), foodPositions[0].getY());

        if(closestFoodDist < 4.0)
          moveSpeed = (int)(closestFoodDist + 0.5);
      }
    }
		else // Attempt to reduce local minima problems.
	  {
			if(moveDirection == Constants.DirectionNorth && previousMoveDirection == Constants.DirectionSouth)
			{
				moveDirection = Constants.DirectionWest;
			}
			else if(moveDirection == Constants.DirectionSouth && previousMoveDirection == Constants.DirectionNorth)
			{
				moveDirection = Constants.DirectionWest;
			}
			else if(moveDirection == Constants.DirectionEast && previousMoveDirection == Constants.DirectionWest)
			{
				moveDirection = Constants.DirectionSouth;
			}
			else if(moveDirection == Constants.DirectionWest && previousMoveDirection == Constants.DirectionEast)
			{
				moveDirection = Constants.DirectionSouth;
      }
    }

    if(shouldAttack)
    {
/*      myColor[0] = 255;
      myColor[1] = 0;
      myColor[2] = 0;
*/
      shouldAttack = false;
      
      double d = Math.sqrt(attackPosition.getX() * attackPosition.getX() + attackPosition.getY() * attackPosition.getY());
      if(d <= 1.0)
      {
        attack();
      }
      else
      {
        int direction = getMyDirection(attackPosition.getX(), attackPosition.getY());

        if(d > 4.0)
          d = 4.0;

				move(direction, (int)d);
      }
    }
    else
    {
/*	    myColor[0] = 0;
	    myColor[1] = 255;
	    myColor[2] = 127;
*/
        int a;
      if(canEat())
        eat();
      else if(getEnergyLevel() > Configuration.AgentMateCost * 1.5)
        mate();
      else
        move(moveDirection, moveSpeed);
    }
		
    previousMoveDirection = moveDirection;
  }

  /* (non-Javadoc)
   * @see yaes.application.eel6938.game.objects.Agent#getName()
   */
  public String getName()
  {
    return myAgentName;
  }

  /* (non-Javadoc)
   * @see yaes.application.eel6938.game.objects.Agent#getColor()
   */
  public int[] getColor()
  {
    return(myColor);
  }

  public int getMyDirection(double x, double y)
	{
		int direction = Constants.DirectionSouth;
		
		if(Math.abs(x) > Math.abs(y))
		{
			if(x > 0.0)
				direction = Constants.DirectionEast;
			else
				direction = Constants.DirectionWest;	
		}
		else
		{
			if(y > 0.0)
				direction = Constants.DirectionNorth;
			else
				direction = Constants.DirectionSouth;	
		}
		
		return(direction);
	}
}