package yaes.application.marketgrid.marketmakers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import yaes.application.marketgrid.MarketMaker;
import yaes.application.marketgrid.brokers.Offer;
import yaes.application.marketgrid.marketmakers.MarketMakerAllocation;
import yaes.application.marketgrid.money.Bank;
import yaes.ui.text.TextUi;

public class SimpleInvestmentStrategy {

  /**
      Investment goal. Defined as percentage of total assets that has to be kept as
      liquid asset i.e. money in the bank. Investment cannot exceed it
   */
  private double liquidAssetsTreshold = 1.0;
  /**
      Trigger at which investment has to occur. Forces allocation of resources.
      It has to be
   */
  private double investRatioTreshold = 10;

  /**
      @return Trigger at which investment has to occur. Forces allocation of resources.
   */
  public double getInvestRatioTreshold(){
    return investRatioTreshold;
  }
  /**
      @parameter investRatioTreshold Trigger at which investment has to occur.
                Forces allocation of resources. It has to be greater than liquidAssetsTreshold
      @return succes of operation
   */
  public boolean setInvestRatioTreshold(double investRatioTreshold){
    if(investRatioTreshold <= liquidAssetsTreshold)
      return false;
    this.investRatioTreshold = investRatioTreshold;
    return true;
  }
  /**
      @return liquidAssetsTreshold Investment goal. Defined as percentage of
                 total assets that has to be kept as liquid asset i.e. money in
                 the bank.
   */
  public double getLiquidAssetsTreshold(){
    return liquidAssetsTreshold;
  }
  /**
      @parameter liquidAssetsTreshold Investment goal. Defined as percentage of
                 total assets that has to be kept as liquid asset i.e. money in
                 the bank. Investment cannot exceed it.
                 Must be positive value and less than investRatioTreshold
      @return succes of operation
   */
  public boolean setLiquidAssetsTreshold(double liquidAssetsTreshold){

    if(investRatioTreshold <= liquidAssetsTreshold ||
       liquidAssetsTreshold < 0.0)
      return false;
    this.liquidAssetsTreshold = liquidAssetsTreshold;
    return true;
  }
  /**
      Advisory function that yields amount to be invested
      @parameter moneyInBank liquid asset
      @parameter resources current value of allocated assets
   */
  public double invest(int moneyInBank, Collection<MarketMakerAllocation> resources){
    if(moneyInBank <= 0)
      return 0.0;
    double liquid_assets = Double.valueOf(moneyInBank);
    double total_assets = liquid_assets + getInvestedValue(resources);
    double liquidRatio =   liquid_assets * 100.0 / total_assets;
    if(liquidRatio >= investRatioTreshold){
      double money_to_keep = total_assets * liquidAssetsTreshold / 100.0;
      return liquid_assets - money_to_keep;
    }
    return 0.0;
  }
  /**
      Advisory function that filters offers to set to be invested
      Offers are sorted descending by total profit they bring
        profit = (highest bid - bidding price) * quantity for the bid
      and bidding quantity
      Affordable offers are accepted until money pool is exhausted.

      @parameter providersOffers
      @parameter moneyInBank liquid asset
      @parameter resources current value of allocated assets
   */
  public void filterOffers(List<Offer> providersOffers, int moneyInBank,
                            Collection<MarketMakerAllocation> resources){
    double money = invest(moneyInBank, resources);
    OfferComparator cmp = new OfferComparator();
    Offer maxPrice = Collections.max(providersOffers, cmp);
    double max_price = maxPrice.allocationDescriptor.expectedUnitPrice;
    ArrayList<OfferEval> evals = new ArrayList<OfferEval>();
    for(Offer offer : providersOffers){
      OfferEval eval = new OfferEval();
      evals.add(eval);
      eval.offer = offer;
      if(offer.allocationDescriptor.quantity != 0.0){
        calc_max(eval, offer.allocationDescriptor.quantity,
                 offer.allocationDescriptor.expectedUnitPrice, max_price);
        calc_min(eval, offer.allocationDescriptor.quantity,
                 offer.allocationDescriptor.expectedUnitPrice, max_price);
      }
      else{
        calc_max(eval, offer.allocationDescriptor.quantityUpperBound,
                 offer.allocationDescriptor.expectedUnitPrice, max_price);
        calc_min(eval, offer.allocationDescriptor.quantityLowerBound,
                 offer.allocationDescriptor.expectedUnitPrice, max_price);
      }
    }
    providersOffers.clear();
    Collections.sort(evals, new OfferEvalComparator());
    for(OfferEval eval : evals){
      // is offer affordable
      if(money > eval.min_invest){
        if(money >= eval.max_invest){
          eval.offer.allocationDescriptor.quantity =
                                            eval.offer.allocationDescriptor.quantityUpperBound;
          money -= eval.max_invest;
        }
        else{
          double value_diff = money - eval.min_invest;
          double quant_diff = value_diff / eval.offer.allocationDescriptor.expectedUnitPrice;
          if(eval.offer.allocationDescriptor.quantityBatchSize > 0.0)
            quant_diff = (double)((int)(quant_diff /
                                            eval.offer.allocationDescriptor.quantityBatchSize));
          eval.offer.allocationDescriptor.quantity =
                eval.offer.allocationDescriptor.quantityLowerBound + quant_diff;
          value_diff = quant_diff * eval.offer.allocationDescriptor.expectedUnitPrice;
          money -= eval.min_invest + value_diff;
        }
        eval.offer.allocationDescriptor.quantityUpperBound = 0.0;
        eval.offer.allocationDescriptor.quantityLowerBound = 0.0;
        // save offer as accepted
        providersOffers.add(eval.offer);
      }

      if(money == 0.0)
        break;
    }
  }

  private void calc_max(OfferEval eval, double quantity, double price, double max_price){
      eval.max_invest = quantity * price;
      eval.max_diff = quantity * (max_price - price);
  }

  private void calc_min(OfferEval eval, double quantity, double price, double max_price){
      eval.min_invest = quantity * price;
      eval.min_diff = quantity * (max_price - price);
  }

  private double getInvestedValue(Collection<MarketMakerAllocation> resources){
    double value = 0.0;
    for(MarketMakerAllocation item : resources){
      if(item.getAvailableQuantity() > 0.0){
        value += item.getAvailableQuantity() * item.getAllocatePrice();
      }
    }

    return value;
  }

  //
  //  --------------    H E L P E R S  -----------------------------------
  //
  public Comparator getOfferComparator(){
    return new OfferComparator();
  }
  public Comparator getOfferEvalComparator(){
    return new OfferEvalComparator();
  }

  private static int cmp(double a, double b){
        if(a < b)
          return -1;
        else if(a == b)
          return 0;
        else
          return 1;
  }

  private class OfferComparator implements Comparator<Offer> {
    public int compare(Offer o1, Offer o2){
      int cmp_result = cmp(o1.allocationDescriptor.expectedUnitPrice,
                          o2.allocationDescriptor.expectedUnitPrice);
      if(cmp_result != 0)
        return cmp_result;
      // compare quantities if both are given as single quantities
      // preference is larger quantity
      if(o1.allocationDescriptor.quantity != 0.0 &&
         o2.allocationDescriptor.quantity != 0.0)
           return cmp(o2.allocationDescriptor.quantity, o1.allocationDescriptor.quantity);

      // compare ranges if both are given as ranges : preference is larger quantity
      if(o1.allocationDescriptor.quantity == 0.0 &&
         o2.allocationDescriptor.quantity == 0.0)
         return cmp(o1.allocationDescriptor.quantityUpperBound,
             o2.allocationDescriptor.quantityUpperBound);
      // single quantity and range are not comparable
      return 0;
    }

  }

  private class OfferEval{
    Offer offer;
    double max_invest = 0.0;
    double min_invest = 0.0;
    double max_diff = 0.0;
    double min_diff = 0.0;
  }

  private class OfferEvalComparator implements Comparator<OfferEval> {
    // Collections.sort sorts in ascending order, so invert comparison!!
    public int compare(OfferEval o1, OfferEval o2){
      int cmp_result = cmp(o2.max_diff, o1.max_diff);
      if(cmp_result != 0)
        return cmp_result;
      return cmp(o2.max_invest, o1.max_invest);
    }
  }
}
