package yaes.application.marketgrid.providers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Hashtable;

import yaes.application.marketgrid.clients.Task;
import yaes.application.marketgrid.money.Bank;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.resource.IResource;
import yaes.application.marketgrid.resource.simple.SimplePricePolicy;
import yaes.application.marketgrid.resource.simple.SimplePricingRule;
import yaes.application.marketgrid.resource.simple.SimpleResource;
import yaes.framework.agent.ExtendedMessage;
import yaes.framework.world.World;

/**
 * This is a version of the provider which allows for multiple allocations from
 * a common set of resources. It does this by handling multiple conversations.
 *
 * @author Lotzi Boloni
 *
 */
public class MultiProvider extends Provider {

  private SimplePricingRule priceRule;
  private SimplePricePolicy pricePolicy;

  private Hashtable<String, ProviderConversation> conversations;
  private ArrayList<TaskRunner> tasksToRun;
  private ProviderAllocation resourceAllocations;
  private ProviderExecutionManager providerExecutionManager;

  public MultiProvider(String name, World world, double initResource, int initMoney) {
    super(name, world, initMoney);
    conversations = new Hashtable<String, ProviderConversation>();
    tasksToRun = new ArrayList<TaskRunner>();
    providerExecutionManager = new ProviderExecutionManager();

    //@FIXME put proper price formation
    priceRule = new SimplePricingRule(1.0);
    pricePolicy = new SimplePricePolicy(priceRule);

    resourceAllocations = new ProviderAllocation(this);
    createResource(initResource, "_RESOURCE_0");
  }
  /**
      Creates resources for provider
   */
  public void createResource(double initResource, String resourceId){

    SimpleResource resource = new SimpleResource();
    resource.setTotalQuantity(initResource);
    resource.setResourceId(resourceId);
    resourceAllocations.addResource(resource);
    providerExecutionManager.addResource(resource);
  }
  /**
      #see yaes.framework.agent.IAgent
   */
  public void action() {
      while(true) {
              ExtendedMessage message = getExtendedMessage();
              if (message == null) {
                break;
              }
        ProviderConversation conversation = conversations.get(message.getConversation());
        if(conversation == null){
          conversation = ProviderConversation.getProviderConversationsInstance(this, message);
          conversations.put(message.getConversation(), conversation);
        }
        conversation.action(message);
      }

      runTasks();

      // clean up the terminated conversations
      for (Enumeration<String> iter = conversations.keys(); iter.hasMoreElements();) {
        String key = iter.nextElement();
        ProviderConversation element = conversations.get(key);
        if (element.isFinished()) {
          conversations.remove(key);
        }
      }
  }

  /**
   * @return Returns price of requested resource.
   */

  public double getPriceOffer(double quantity){
    return pricePolicy.getPrice(quantity);
  }
  /**
      Initiates execution of the task using commited resources
   */
  public boolean addTaskToRunCommited(Task task, String conversationId, String client){
    if(! tryLockResource(task.getRequiredResources(), conversationId, client))
      return false;
    IResource resource = providerExecutionManager.getResourceForTask(task);
    if(resource == null){
      releaseResource(conversationId);
      return false;
    }

    tasksToRun.add(new TaskRunner(task, this, conversationId, client, resource, true));
    return true;
  }
  /**
      Initiates execution of the task using free (non-commited) resources
   */
  public boolean addTaskToRunFree(Task task, String conversationId, String client){

    if(! tryLockResource(task.getRequiredResources(), conversationId, getName()))
      return false;

    IResource resource = providerExecutionManager.getResourceForTask(task);
    if(resource == null){
      releaseResource(conversationId);
      return false;
    }

    tasksToRun.add(new TaskRunner(task, this, conversationId, client, resource, false));
    return true;
  }

  private boolean tryLockResource(AllocationDescriptor requiredResources,
                    String conversationId, String owner){

    if(requiredResources.quantityUpperBound == 0.0)
      requiredResources.quantityUpperBound = requiredResources.quantity;
    if(requiredResources.quantityLowerBound == 0.0)
      requiredResources.quantityLowerBound = requiredResources.quantity;

    // try to lock upper bound first
    if( resourceAllocations.lockResource(requiredResources.quantityUpperBound,
                                          owner, conversationId))
      return true;
    return resourceAllocations.lockResource(requiredResources.quantityLowerBound,
                                          owner, conversationId);
  }

  private void runTasks(){
    for(Iterator<TaskRunner> it=tasksToRun.iterator(); it.hasNext();){
      TaskRunner task = it.next();
      task.action();
      if(task.hasTerminated())
        it.remove();
    }
  }
  /**
      Allocates commited resource
   */
  public boolean allocateCommited(AllocationDescriptor requiredResources, String user){
    return resourceAllocations.allocateCommited(requiredResources.quantity, user);
  }
  /**
      Releases locked resource
   */
  public void releaseResource(String lock){
    resourceAllocations.unlockResource(lock);
  }
  /**
      Makes wholesale offer to marketmakers
   */
  //@FIXME this is dummy. Make meaningful version
  public Collection<AllocationDescriptor> getWholeSaleOffer(
                                          AllocationDescriptor requiredResources){

    double available = resourceAllocations.getFreeResourcesQuantity();
    if(available == 0.0)
      return null;
    Collection<AllocationDescriptor> offers = new ArrayList<AllocationDescriptor>();
    offers.add(requiredResources);
    requiredResources.quantityLowerBound = 0;
    requiredResources.quantityUpperBound = available;
    requiredResources.expectedUnitPrice =  pricePolicy.getPrice(available) / available;
    return offers;
  }
  /**
      Charges use of free (direct retail) resources that are consumed
   */
  public void chargeResource(String client, AllocationDescriptor requiredResources){
    int value = (int)pricePolicy.getPrice(requiredResources.quantity);
    Bank.getInstance().transfer(client, getName(), value);
  }
  /**
      Charges use of resources that are commited after being consumed
   */
  public void chargeCommitedResource(String client, AllocationDescriptor requiredResources){
    //@FIXME resolve charging
  }
}
