package yaes.application.marketgrid.resource;

import java.util.Collection;


public interface IResource{
  /**
    Description of the resource
 */
  public double getAvailableQuantity();

/**
 * Request to allocate resource. Resource implementation provides specific logic
 * @param requestedQuantity - value of units apropriate for the resource
 * @param user - user instance that requested allocation (assumed that resource will keep
 *               track of allocations and users
 * @param fragment permits allocation of fragment of quantity if entire cannot
 *               be accomodated
 * @return actually allocated quantity (which may be zero if request failed
 */
  public double allocate(double requestedQuantity, String user, boolean fragment);
/**
 * Request to allocate resource. Resource implementation provides specific logic
 * @param requestedQuantity - instance of UsageChunk that describes value of units
 *        apropriate for the resource, period of allocation and user
 * @param fragment permits allocation of fragment of quantity if entire cannot
 *               be accomodated
 * @return UsageChunk with actually allocated quantity (which may be zero if request failed
 */
  public UsageChunk allocate(UsageChunk requestedQuantity, boolean fragment);

/**
 * Request to release allocated resource. Resource implementation provides specific logic
 * @param releaseQuantity - value of units apropriate for the resource
 * @param user - user instance that requested release (assumed that resource will keep
 *               track of allocations and users. Implementation has to solve conflicts
 *               that may rise out of difference in allocation and release request)
 */
  public void release(double releaseQuantity, String user);

/**
 * Request to release allocated resource. Resource implementation provides specific logic
 * @param releaseQuantity - instance of UsageChunk that describes value of units
 *        apropriate for the resource, period of allocation and user
 *        that requested release (assumed that resource will keep
 *        track of allocations and users. Implementation has to solve conflicts
 *        that may rise out of difference in allocation and release request)
 */
  public void release(UsageChunk releaseQuantity);

/**
 * Resource type that defines class of resources as type (memory, processor, ...)
 * @return type of resource being queried
 */
  public String getResourceType();

/**
 * Resource identification that defines resource as instance (usually among
 * instances of the same type)
 * @return identification string of resource being queried
 */
  public String getResourceId();

/**
 * Sets rules that define cost calculation
 * @param rules - rules for cost calculus apropriate for the resource
 *                (resource instance?)
 */
  public void setPricingPolicy(Collection<IPricingRule> rules);

/**
 * Retrieves rules that define cost calculation
 * @return  rules for cost calculus apropriate for the resource instance
 */
  public Collection<IPricingRule> getPricingPolicy();

/**
 * Retrieves set of distingushing parameters that describe resource
 * (e.g. processor type, processor speed)
 * NOTE: this is not quantity that is allocated
 * @return set of distingushing parameters that describe resource instance
 */
  public Collection<ResourceParameter> getResourceParameters();

/**
 * Sets set of distingushing parameters that describe resource
 * (e.g. processor type, processor speed)
 * NOTE: this is not quantity that is allocated
 * @param distingushingParameters - set of distingushing parameters that describe resource instance
 */
  public void setResourceParameters(Collection<ResourceParameter> distingushingParameters);

/**
 * Retrieves set of resources that are bound together
 * (e.g. processor, memory) in order to prevent nonfeasible divisions
 * @return set of bound resource instances
 */
  public Collection<IResource> getResourceBindings();
/**
 * Sets set of resources that are bound together
 * (e.g. processor, memory) in order to prevent nonfeasible divisions
 * @param boundResources - set of bound resource instances
 */
  public void setResourceBindings(Collection<IResource> boundResources);
}
