package yaes.application.uavaltclient.agents;

import java.util.ArrayList;

import yaes.application.uavaltclient.message.MessageLibrary;
import yaes.application.uavaltclient.message.MsgType;
import yaes.application.uavaltclient.missionmgr.MissionManager;
import yaes.application.uavaltclient.nav.NavUtil;
import yaes.application.uavaltclient.nav.Pos;
import yaes.application.uavaltclient.simulation.SimException;
import yaes.application.uavaltclient.util.Props;
import yaes.framework.agent.GenericMessage;
import yaes.framework.world.World;


/**
 * Scout uav, capable of spotting ground targets.
 * 
 * @author dejongp
 */
public class Scout extends UAV
{
	// Scout configuration constants (Scout is most navigable UAV) 
	private static final double UAV_MAX_HDG_DELTA = Props.getDouble("SCOUT_UAV_MAX_HDG_DELTA"); //max heading change per tick (degrees)	
	private static final double UAV_MAX_ALT_DELTA = Props.getDouble("SCOUT_UAV_MAX_ALT_DELTA"); //(OpenGL coordinates/tick)

	private ArrayList bombProposals; //storage container for leader proposals during negotiation
	private int numBombProposalsExpected = 0;
	private Pos targetPos = null;
	
	private ArrayList targetsFound = new ArrayList(); // list of targets found, so they are ignored when found multiple times
	
	/**
	 * @param a
	 */
	public Scout(String name, World world, double firepower, double visibility, double maxVelocity, double stealth) 
	{
		super(name, world, firepower, visibility, maxVelocity, stealth);
  
		// initialize this UAV with the simulation
		try
		{ 
			type = UAVType.UAV_TYPE_SCOUT;
			sim.init(name, type);
		}
		catch(SimException e) 
		{
			System.err.println("COULD NOT REGISTER " + name + " WITH SIMULATION: "+ e);
			e.printStackTrace();
		}
	}
	
	/**
	 * Actions unique to scout vehicles.
	 * 
	 * @see yaes.framework.agent.IAgent#action(yaes.framework.world.World)
	 */
	public void action()
	{
		super.action();

	  try
		{
			Pos pos = sim.enemy();
			if (pos != null && !targetAlreadyFound(pos))
			{
				for (int i=0; i<squadronMembers.length; i++)
					if (! name.equals(squadronMembers[i]))
						MessageLibrary.sendBomberNeededMsg(name, squadronMembers[i]);
						
				bombProposals = new ArrayList();
				bombNegotiationFlag = true;
				numBombProposalsExpected = squadronMembers.length - 1;
				targetPos = pos;
				targetsFound.add(targetPos.clone());
			}
		}
		catch (SimException e) {}
		
	}
	
	
	/**
	 * Obtain the maximum heading change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxHdgDelta()
	{
		return UAV_MAX_HDG_DELTA;
	}	
	
	/**
	 * Obtain the maximum altitude change per tick for this UAV.
	 * 
	 * @return max heading change per tick (degrees)
	 */
	public double getMaxAltDelta()
	{
		return UAV_MAX_ALT_DELTA;
	}	
	  
	/**
	 * Req'd by UAV superclass.  Processes messages not handled by superclass.
	 *
	 * @param msg the incoming GenericMessage message
	 * @param type the type of the message (from MsgType)
	 */  
	protected void processMessage(GenericMessage msg, String msgType)
	{
		// UAV receives a bomb proposal as part of a bomb contract net negotiation
		if (MsgType.MSG_PROPOSE_BOMB.equals(msgType))
		{
			bombProposals.add(msg);
			numBombProposalsExpected --;
			if (numBombProposalsExpected == 0)
				selectBomber();
		}
		
		// UAV receives bomb refusal as part of a bomb contract net negotiation
		else if (MsgType.MSG_DECLINE_BOMB.equals(msgType))
		{
			numBombProposalsExpected --;
			if (numBombProposalsExpected == 0)
				selectBomber();
		}
		
		// respond to a bomber negotiation call-for-proposal
		else if (MsgType.MSG_BOMBER_NEEDED.equals(msgType))
		{
			bombNegotiationFlag = true;
		  MessageLibrary.sendDeclineBombMsg(name, (String) msg.getValue("sender"));  
		} 		
		
		else
		  System.err.println("MESSAGE TYPE NOT RECOGNIZED: " + msgType);
	}

	/**
	 * Select a bomber from the candidates as part of a bomber negotiation.
	 * Precondition: all proposals have been received.
	 */
  private void selectBomber()
  {	
		String bestCandidate = null;
		Pos bestPos = null;
		
		for (int i=0; i<bombProposals.size(); i++)
		{
			GenericMessage proposalMsg = (GenericMessage) bombProposals.get(i);

			Pos proposedPos = (Pos) proposalMsg.getValue("pos");
			
			if ((bestCandidate == null) ||
			    (NavUtil.distance(proposedPos.getX(), proposedPos.getZ(), targetPos.getX(), targetPos.getZ()) < 
		       NavUtil.distance(bestPos.getX(), bestPos.getZ(), targetPos.getX(), targetPos.getZ())))
		  {
		  	bestCandidate = (String) proposalMsg.getValue("sender");
				bestPos = proposedPos;    
		  }
		}
				
		if (bestCandidate == null)
		{
			System.out.println("No candidates for bombing.  Continuing with mission.");
		}
    else
    {
			for (int i=0; i<squadronMembers.length; i++)
			{
				// assign uav to wait while bombing is carried out
				if ((! squadronMembers[i].equals(bestCandidate)) &&
				    (! squadronMembers[i].equals(name)))
				  MessageLibrary.sendAssignWaitMsg(name, squadronMembers[i]);
	
				// send bomb assignment
				else
				  MessageLibrary.sendAssignBombMsg(name, squadronMembers[i], targetPos);
			}
			
			// this scout is not a bombing candidate, and therefore must assign itself to wait
			mgr.setWaitPosition(mgr.getCurrentPosition());
			mgr.setCurrentMode(MissionManager.MODE_WAIT);
    }
			
		// exit bomb negotiation state
		bombNegotiationFlag = false;
		numBombProposalsExpected = 0;
		targetPos = null;
		bombProposals = null;  	
  }
	
	/**
	 * Determines if a target has already been found by this uav.
	 * 
	 * @param pos position of target
	 * @return true if target was previously found, false otherwise
	 */
	private boolean targetAlreadyFound(Pos pos)
	{
		for (int i=0; i<targetsFound.size(); i++)
		{
			Pos previousTarget = (Pos) targetsFound.get(i);
			if (NavUtil.isPosReached(pos, previousTarget))
				return true;
		}
		
		return false;
	}
	
	/**
	 * Reset this list of discovered targets.  This should be performed when squadron configuration
	 * changes, since targets which were ignored because no bomber was present may now contain a
	 * bomber.
	 *
	 */
	void resetTargetsFound()
	{
		targetsFound = new ArrayList();
	}
	
}