package yaes.application.uavaltclient.coalitionformation;



import java.math.BigInteger;

import java.util.HashMap;

import java.util.Map;



/**

 * Generates combinations using a depth-first search algorithm.  Each item is represented as a

 * number from 0 to (n-1) where n is the size of the combination element set.  Results are 

 * returned in an array.  "Bad branches" can be specified, causing search paths based on a

 * specified combination to be ignored.

 * 

 * @author dejongp

 */

public class DepthFirstAlgorithm

{

	protected int n; //number of items to generate combination of

	protected int[] ar; //working array holding current combination data

	protected int in = -1; //current index

	protected BigInteger counter = BigInteger.ZERO;

	private Map badBranches = new HashMap();

		

	/**

	 * For testing

	 * 

	 * @param args

	 */

	public static void main(String args[])

	{

		DepthFirstAlgorithm df = new DepthFirstAlgorithm(8);

		//df.addBadBranch(new int[]{7});

		//df.addBadBranch(new int[]{5,7});

		//df.addBadBranch(new int[]{1,2,3,7});



		int[] result = df.nextCombo();

		

		while (result != null)

		{

			printCombination(result);

			result = df.nextCombo();

		}

		

		System.out.println("NUMBER COMBINATIONS: " + df.getCombinationCount().toString());

	}

	

	/**

	 * Constructor

	 * 

	 * @param n number of items to generate combinations of

	 */

	public DepthFirstAlgorithm(int n)

	{

		this.n = n;

		ar = new int[n];

	}

	

	/**

	 * Generate the next combination.

	 * 

	 * @return int array containing next combination, or null if search is complete

	 */

	public int[] nextCombo()

	{

		nextIndex();

		if (in == -1) return null;



		ar[in] ++;

		

		while (isBadBranch() ) 

			if (! nextBranch())

				return null;

	

		counter = counter.add(BigInteger.ONE);

		

		int[] retarray = new int[in+1];

		System.arraycopy(ar, 0, retarray, 0, in+1);

		return retarray;

	}

	

	/**

	 * "Pops" working array until an element is found which can be incremented. 

	 * 

	 *  postcondition: in will be set to next index to increment, or -1 if search is complete.

	 */

	private void nextIndex()

	{	

		// first iteration

		if (in == -1)	

		{

			ar[++in] = -1;

			return;

		}

		

		// deepen

		if ((in < (n-1)) && (ar[in] < (n-1)))

		{ 

			ar[++in] = ar[in - 1];

		  return;

		}

		

		// backtrack

		for (; in>=0; in--)

		{

			if (ar[in] < (n-1))

				return;

		}

		

		// search complete

		in = -1;

	}



	/**

	 * Display a combination array

	 * 

	 * @param combination the array of combination indexes returned by getNext()

	 */

	public static void printCombination(int[] combination)

	{

		for (int i=0; i<combination.length; i++)

			System.out.print(combination[i] + " ");

			

		System.out.println();

	}

		

  /**

   * @return number of good combinations which were generated

   */

  public BigInteger getCombinationCount()

  {

    return counter;

  }

  

	

	/**

	 * User marks a new root combination as not capable of producing acceptable coalitions.

	 * (This will work for n <= 99)

	 * 

	 * @param combination the array of combination indexes returned by getNext()

	 */

	public void addBadBranch(int[] combination)

	{

		String rootKey = "";

		

		for (int i=0; i<combination.length; i++)

		{

			if (combination[i] < 10)

				rootKey += "x" + combination[i];

			else

				rootKey += combination[i];

		}



		badBranches.put(rootKey, null);

		//System.out.println("Added bad branch: " + rootKey);

	}

	

	/**

	 * Returns true if this combination or subarray with size (n-1) is contained 

	 * in badBranches map, false otherwise.

	 * 

	 * @param combination

	 * @param numElements number of elements of combination array to consider as a single array (other elements will be ignored)

	 * @return true if combination is contained in badRoots map, false otherwise

	 */

	protected boolean isBadBranch()

	{

		String rootKey = "";

		

		for (int i=0; i<=in; i++)

		{

			if (ar[i] < 10)

				rootKey += "x" + ar[i];

			else

				rootKey += ar[i];

		}

		

		String smRootKey = rootKey.substring(0, rootKey.length() - 2);

		boolean isBadBranchFlag = false;

		

		if (badBranches.containsKey(smRootKey))

		{

			isBadBranchFlag = true;

			//System.out.println("SM BAD BRANCH FOUND: " + smRootKey + " in " + rootKey);		

			in --;

		}

		

		else if (badBranches.containsKey(rootKey ))

		{

			isBadBranchFlag = true;

			//System.out.println("BAD BRANCH FOUND: " + rootKey);

		}

			

		return isBadBranchFlag;

	}

	

	/**

	 * Generates next branch in sequence.

	 *

	 * @return true if next branch was generated, false if not possible

	 */

	protected boolean nextBranch()

	{

		// increment in place

		if (ar[in] < (n-1))

			ar[in] ++;

		

		else

		{			

			// backtrack

			for (; in>=0; in--)

			{

				if (ar[in] < (n-1))

				{

					ar[in] ++;

					break;

				}

			}

		}

		

		// end of iteration

		if (in == -1)

	    return false;

	    

	  return true;

	}

}

