package yaes.application.uavaltclient.message;



import yaes.application.uavaltclient.main.Main;

import yaes.application.uavaltclient.missionmgr.Mission;

import yaes.application.uavaltclient.missionmgr.Objective;

import yaes.application.uavaltclient.nav.Pos;

import yaes.framework.agent.Directory;

import yaes.framework.agent.GenericMessage;





/**

 * Library containing message sending functions used for HQ-->UAV and UAV-->UAV communication.

 * 

 * @author dejongp

 */

public class MessageLibrary

{

	private static Directory yaesDirectory = Main.getYaesDirectory();

	

	/**

	 * Assign a new mission to the receiver.

	 *

	 * MSG contents:

	 *   type: MSG_NEW_MISSION

	 *   mission: [Mission object] to assign

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param mission new Mission assignment

	 */	

	public static void sendNewMissionMsg(String sender, String receiver, Mission mission, Boolean initialAssignment)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_NEW_MISSION);

		msg.setValue("mission", mission);

		msg.setValue("initialAssignment", initialAssignment);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Request that the current mission be returned to the sender.

	 *

	 * MSG contents:

	 *   type: MSG_REQUEST_MISSION

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendRequestMissionMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_REQUEST_MISSION);

		yaesDirectory.deliverMessage(msg);		

	}



	/**

	 * Send the current mission to the receiver in response to a request.

	 *

	 * MSG contents:

	 *   type: MSG_MISSION

	 *   mission: [Mission object] which was requested

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param mission new Mission assignment

	 * @param objectiveIndex indicates which objective is the current one in the mission

	 */	

	public static void sendMissionMsg(String sender, String receiver, Mission mission, Integer objectiveIndex)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_MISSION);

		msg.setValue("mission", mission);

		msg.setValue("objectiveIndex", objectiveIndex);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Assign a new squadron to the recipient.

	 *

	 * MSG contents:

	 *   type: MSG_NEW_SQUADRON

	 *   squadname: [String object] name of the squadron the uav is assigned to

	 *   members: [String[] ] array of uav names in the squadron

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param squadron name of the squadron (unique agent name)

	 * @param members String array of names of uavs in this squadron

	 */	

	public static void sendNewSquadronMsg(String sender, String receiver, String squadron, String[] members)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_NEW_SQUADRON);

		msg.setValue("squadron", squadron);

		msg.setValue("members", members);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Request that the current objective be returned to the sender.

	 *

	 * MSG contents:

	 *   type: MSG_GET_CURRENT_OBJECTIVE

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendGetCurrentObjectiveMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_GET_CURRENT_OBJECTIVE);

		yaesDirectory.deliverMessage(msg);		

	}	

	

	/**

	 * Return the current objective in response to a request.

	 *

	 * MSG contents:

	 *   type: MSG_CURRENT_OBJECTIVE

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param objective current Objective of the sender

	 */	

	public static void sendCurrentObjectiveMsg(String sender, String receiver, Objective objective)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_CURRENT_OBJECTIVE);

		msg.setValue("objective", objective);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Request that a string description of all objectives be returned to the sender.

	 *

	 * MSG contents:

	 *   type: MSG_GET_OBJECTIVES

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendGetObjectivesMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_GET_OBJECTIVES);

		yaesDirectory.deliverMessage(msg);		

	}	

	

	/**

	 * Return a string description of all objectives in a mission in response to a request.

	 *

	 * MSG contents:

	 *   type: MSG_OBJECTIVES

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param objectivesDescription String describing current Objectives

	 */	

	public static void sendObjectivesMsg(String sender, String receiver, String objectivesDescription)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_OBJECTIVES);

		msg.setValue("objectivesDescription", objectivesDescription);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Initiate a leader negotiation.

	 *

	 * MSG contents:

	 *   type: MSG_LEADER_NEEDED

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendLeaderNeededMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_LEADER_NEEDED);

		yaesDirectory.deliverMessage(msg);		

	}	



	/**

	 * Sent by a uav proposing squadron leadership.  Part of leader negotiation.

	 *

	 * MSG contents:

	 *   type: MSG_PROPOSE_LEAD

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param pos current position of the sender

	 * @param uavType this uav's type (one of: UAVType)

	 */	

	public static void sendProposeLeadMsg(String sender, String receiver, Pos pos, String uavType)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_PROPOSE_LEAD);

		msg.setValue("pos", pos);

		msg.setValue("uavType", uavType);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Assigns this uav to be formation leader.

	 *

	 * MSG contents:

	 *   type: MSG_ASSIGN_LEADER

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendAssignLeaderMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_ASSIGN_LEADER);

		yaesDirectory.deliverMessage(msg);

	}		

	

	/**

	 * Assigns this uav to a formation slot behind a leader.

	 *

	 * MSG contents:

	 *   type: MSG_ASSIGN_FORMATION

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

   * @param leader name of uav leading the formation

   * @param row row number behind the leader

   * @param side "left" or "right" of the leader

   */

	public static void sendAssignFormationMsg(String sender, String receiver, String leader, 

	                                          Integer row, String side)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_ASSIGN_FORMATION);

		msg.setValue("leader", leader);

		msg.setValue("row", row);

		msg.setValue("side", side);

		yaesDirectory.deliverMessage(msg);

	}			

	

	/**

	 * Initiate a bomber negotiation.

	 *

	 * MSG contents:

	 *   type: MSG_BOMBER_NEEDED

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendBomberNeededMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_BOMBER_NEEDED);

		yaesDirectory.deliverMessage(msg);		

	}	



	/**

	 * Sent by a uav proposing bomber in a bomb run.  Part of bomber negotiation.

	 *

	 * MSG contents:

	 *   type: MSG_PROPOSE_BOMB

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param pos current position of the sender

	 */	

	public static void sendProposeBombMsg(String sender, String receiver, Pos pos)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_PROPOSE_BOMB);

		msg.setValue("pos", pos);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Send refusal in response to bomb proposal.

	 *

	 * MSG contents:

	 *   type: MSG_DECLINE_BOMB

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendDeclineBombMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_DECLINE_BOMB);

		yaesDirectory.deliverMessage(msg);		

	}	

	

	/**

	 * Assigns this uav to be the bomber in a bomb run.

	 *

	 * MSG contents:

	 *   type: MSG_ASSIGN_BOMB

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 * @param pos position of the target to bomb

	 */	

	public static void sendAssignBombMsg(String sender, String receiver, Pos pos)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_ASSIGN_BOMB);

		msg.setValue("pos", pos);

		yaesDirectory.deliverMessage(msg);

	}		

	

	/**

	 * Instruct uavs to loiter in place while a bomb run is carried out.

	 *

	 * MSG contents:

	 *   type: MSG_ASSIGN_WAIT

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendAssignWaitMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_ASSIGN_WAIT);

		yaesDirectory.deliverMessage(msg);		

	}	

	

	/**

	 * Instruct uavs to resume mission since bomb run is complete.

	 *

	 * MSG contents:

	 *   type: MSG_RESUME_MISSION

	 *  

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message

	 */	

	public static void sendResumeMissionMsg(String sender, String receiver)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_RESUME_MISSION);

		yaesDirectory.deliverMessage(msg);		

	}

	

	/**

	 * Send positional information to other UAVs.

	 * 

	 * MSG contents:

	 *   type: MSG_POS

	 * 

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message* 

	 * @param pos Pos instance containing UAV current positional information

	 */

	public static void sendPosMsg(String sender, String receiver, Pos pos)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_POS);

		msg.setValue("pos", pos);

		yaesDirectory.deliverMessage(msg);

	}	

		

	/**

	 * Send objective number to other UAVs.

	 * 

	 * MSG contents:

	 *   type: MSG_OBJECTIVE_NUM

	 * 

	 * @param sender agent name sending the message

	 * @param receiver agent name receiving the message* 

	 * @param objectiveIndex indicates which objective is the current one in the mission

	 */

	public static void sendObjectiveNumMsg(String sender, String receiver, Integer objectiveIndex)

	{

		GenericMessage msg = new GenericMessage(receiver);

		msg.setValue("sender", sender);

		msg.setValue("type", MsgType.MSG_OBJECTIVE_NUM);

		msg.setValue("objectiveIndex", objectiveIndex);

		yaesDirectory.deliverMessage(msg);

	}			

}

