package yaes.application.uavaltclient.missionmgr;

import yaes.application.uavaltclient.nav.Pos;

/**
 * Models a single objective within a Mission.
 * 
 * @author dejongp
 */
public class Objective implements Cloneable, Comparable
{
  // enumeration of objective types
	public static final int RECON = 1;  // recon within a cube
	public static final int LOITER = 2; // loiter around a point, or within a cube if pos2!=null
	public static final int BOMB = 3;   // bomb at pos1
	// TBD: [. . .] also add entry to getObjectiveValue()
		
  //subtypes for objectives
  
  // loiter varieties:
  public static final int SINGLEPOINT = 11;
  public static final int LAWNMOWER = 12;
		
	private Pos pos1; // postion 1 of objective (NO UNITS) (top left of cube)
	private Pos pos2; // position 2 of objective (NO UNITS) (if appropriate) (bot rt of cube)

  private String name; // a handle for this objective, useful for output
  private int duration; // duration of objective (sec.) (if appropriate)	
	private int type; // objective type (enumerated value from above
  private int subtype;
  
	private int priority; // 1-5, 1==max priority

  private int firepower;
  private int visibility;
  private double velocity;
  private int stealth;
  
  /**
   * Clone this Objective object.
   */
  public Object clone()
  {
		try
		{
			Objective newObjective = (Objective) super.clone();
			
			if (pos1 != null)
				newObjective.pos1 = (Pos) pos1.clone();
				
		  if (pos2 != null)
				newObjective.pos2 = (Pos) pos2.clone();
				
			return newObjective;
		}
		catch (CloneNotSupportedException e)
		{
			throw new InternalError(e.toString());
		}  	
  }


  /**
   * Return string form of this Objective.
   * 
   * @return String form of this objective
   */
  public String toString()
  {
  	String s = "Objective '" + name + "', type: ";
  	
  	switch (type)
  	{
  		case RECON: s += "RECON"; break;
  		case LOITER: s += "LOITER"; break;
  		case BOMB: s += "BOMB"; break;
  	}
  	
  	if (type == LOITER)
  	{
  		if (subtype == SINGLEPOINT) s += ", subtype: SINGLEPOINT";
  		else if (subtype == LAWNMOWER) s += ", subtype: LAWNMOWER";
  	}
  	
  	s += ", Priority: " + priority + "\r\n";
  	
		s += "POS1: " + pos1.toString() + "\r\n";
		
  	if (pos2 != null)
			s += "POS2: " + pos2.toString() + "\r\n";
			
	  if (firepower > 0)
	  	s += "required capability: FIREPOWER: " + firepower + "\r\n";
	  else if (visibility > 0)
	    s += "required capability: VISIBILITY: " + visibility + "\r\n";
		else if (velocity > 0)
			s += "required capability: VELOCITY: " + velocity + "\r\n";
		else if (stealth > 0)
			s += "required capability: STEALTH: " + stealth + "\r\n";			
  	
  	return s;
  }
  
  /**
   * Compares two Objectives (by priority).
   * 
   * @param o1 First object for comparison
   * @param o2 Second object for comparison
   * @return positive int if o1.priority is higher than o2.priority,
   *         0 if o1.priority is same as o2.priority,
   *         negative int if o1.priority is lower than o2.priority
   */
  public int compareTo(Object o) throws ClassCastException
	{
  	int ret = getPriority() - ((Objective) o).getPriority();
	  return ret;
	}
	
	/**
	 * Compares this object to supplied object by priority.
	 * 
	 * @param o Object for comparison with this
	 * @return true if priority of this equals priority of o, false otherwise
	 */
	public boolean equals(Object o)
	{
		try
		{
			return ((Objective)o).getPriority() == getPriority();
		}
		catch (ClassCastException e) 
		{
			return false;
		}
	}
	
  /**
   * Return the int value for the string representation of an objective type enumerated value name
   * 
   * @param objective The string holding the name of an objective type enumerated value name
   * @return the corresponding int value to the objective type enumerated value name
   */
  public static int getObjectiveValue(String objective)
  {
  	int ret = -1;
  	
  	if (objective.equalsIgnoreCase("RECON"))
  	  ret = 1;
  	else if (objective.equalsIgnoreCase("LOITER"))
  	  ret = 2;
  	else if (objective.equalsIgnoreCase("BOMB"))
  	  ret = 3;
  	  
  	// subtypes
		else if (objective.equalsIgnoreCase("SINGLEPOINT"))
			ret = 11;
		else if (objective.equalsIgnoreCase("LAWNMOWER"))
			ret = 12;			
  	
  	return ret;
  }

  /**
   * @return
   */
  public int getDuration()
  {
    return duration;
  }

  /**
   * @return
   */
  public Pos getPos1()
  {
    return pos1;
  }

  /**
   * @return
   */
  public Pos getPos2()
  {
    return pos2;
  }

  /**
   * @return
   */
  public int getType()
  {
    return type;
  }

  /**
   * @param i
   */
  public void setDuration(int i)
  {
    duration = i;
  }

  /**
   * @param pos
   */
  public void setPos1(Pos pos)
  {
    pos1 = pos;
  }

  /**
   * @param pos
   */
  public void setPos2(Pos pos)
  {
    pos2 = pos;
  }

  /**
   * @param i
   */
  public void setType(int i)
  {
    type = i;
  }

  /**
   * @return
   */
  public int getSubtype()
  {
    return subtype;
  }

  /**
   * @param i
   */
  public void setSubtype(int i)
  {
    subtype = i;
  }

  /**
   * @return
   */
  public int getPriority()
  {
    return priority;
  }

  /**
   * @param i
   */
  public void setPriority(int i)
  {
    priority = i;
  }

  /**
   * @return
   */
  public String getName()
  {
    return name;
  }

  /**
   * @param string
   */
  public void setName(String string)
  {
    name = string;
  }

  /**
   * @return
   */
  public int getFirepower()
  {
    return firepower;
  }

  /**
   * @return
   */
  public int getStealth()
  {
    return stealth;
  }

  /**
   * @return
   */
  public int getVisibility()
  {
    return visibility;
  }

  /**
   * @param i
   */
  public void setFirepower(int i)
  {
    firepower = i;
  }

  /**
   * @param i
   */
  public void setStealth(int i)
  {
    stealth = i;
  }

  /**
   * @param i
   */
  public void setVisibility(int i)
  {
    visibility = i;
  }

  /**
   * @return
   */
  public double getVelocity()
  {
    return velocity;
  }

  /**
   * @param d
   */
  public void setVelocity(double d)
  {
    velocity = d;
  }

}
