package yaes.application.uavaltclient.missionmgr;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import yaes.application.uavaltclient.nav.Pos;
import yaes.application.uavaltclient.util.Props;


/**
 * Models a UAV's knowledge of physical space and proximity to other aircraft. Can be used as the 
 * basis of a graphical map representation for a GUI.
 * 
 * @author dejongp
 */
public class UAVMap
{
	public static final int MAP_SIZE = Props.getInt("MAP_SIZE"); // corresponds to MAP_SIZE in simulation
	
	private HashMap aircraft; // Pos[].  Holds known info on other UAVs.
	private HashMap lastAircraft; // Pos[].  Holds previous info on other UAVs.
	private int[][] heightMap; //keeps track of known altitudes 	
	
	private static int instanceCnt = 0;
	private int instance;
	
	/**
	 * Construct a new UAVMap.
	 */
	public UAVMap()
	{
		instanceCnt ++;
		instance = instanceCnt;
		
		aircraft = new HashMap();
		lastAircraft = new HashMap();
		heightMap = new int[MAP_SIZE][MAP_SIZE];
	}
	
	/**
	 * Add an aircraft to the collection of known aircraft which this UAV is keeping track of.
	 * 
	 * @param name Name (AID) of the aircraft to track
	 * @param pos Position of the aircraft to track
	 */
	public void addAircraft(String name, Pos pos)
	{
		// TBD: not sure what else needs to happen here...
		updateAircraft(name, pos);
	}
	
	/**
	 * Update the known position of an aircraft which this UAV is keeping track of.  Also store
	 * previously known position.
	 * 
	 * @param name full aid string of the uav
	 * @param pos current position of uav
	 */
	public void updateAircraft(String name, Pos pos)
	{
		Object lastpos = aircraft.get(name);
		if (lastpos != null) lastAircraft.put(name, lastpos);
		aircraft.put(name, pos);
	}
	
  /**
   * Obtain Positions array for other aircraft (used in soc pot field calculations).
   * 
   * @return position array containing positions of other aircraft
   */
	public Pos[] getPositions(String uav)
	{
		if (! aircraft.isEmpty())
		{
		  Collection posCol = aircraft.values();
		  
		  Pos[] posArray = new Pos[posCol.size()];
		  Iterator iter = posCol.iterator();
		  int i=0;
		  while (iter.hasNext())
		  	posArray[i++] = (Pos) iter.next();

		  return (Pos[]) posArray;
		}
		else
		{
		  return null;
		}
	}
	
	/**
	 * Return current position of specified aircraft.
	 * 
	 * @param name name of aircraft to return position for
	 * @return position of aircraft
	 */
	public Pos getPosition(String name)
	{
		return (Pos) aircraft.get(name);
	}
	
  /**
   * @return
   */
  public int[][] getHeightMap()
  {
    return heightMap;
  }
  
  /**
   * Return height from height map at x,z coordinate, or 0 if the coordinates are invalid.
   * 
   * @param x
   * @param z
   * @return height at x,z
   */
  public int getHeight(int x, int z)
  {
  	try
  	{
  		return heightMap[x][z];
  	}
  	catch(ArrayIndexOutOfBoundsException e)
  	{
  		return 0;
  	}
  }
  
  
  /**
   * Display the map (useful only for debugging).
   */
  public void printMap()
  {
  	for (int col=0; col<heightMap.length; col++)
  	{
  		System.out.println();
  		for (int index=0; index<heightMap[col].length; index++)
  			System.out.print(heightMap[col][index] + " ");
  	}
  }

}
