package yaes.application.uavaltclient.nav;

import javax.vecmath.Vector2d;
import javax.vecmath.Vector3d;

import yaes.application.uavaltclient.missionmgr.UAVMap;
import yaes.application.uavaltclient.util.Props;


/**
 * Static navigation utility functions.
 * 
 * @author dejongp
 */
public class NavUtil
{
  //size of radius around nav point (OpenGL map coordinate units in 3d space)  
	public static final double NAV_DISTANCE_TOLERANCE = Props.getDouble("NAV_DISTANCE_TOLERANCE");
	 
  //(OpenGL map coordinate units)
	public static final double NAV_HEIGHT_TOLERANCE = Props.getDouble("NAV_HEIGHT_TOLERANCE"); 
	
  //(degrees in 2d space) 
	public static final double NAV_HEADING_TOLERANCE = Props.getDouble("NAV_HEADING_TOLERANCE"); 

  //tolerance for slot position (dist in 2d space)
	public static final double NAV_FORMATION_DISTANCE_TOLERANCE = Props.getDouble("NAV_FORMATION_DISTANCE_TOLERANCE"); 
	
	// Note: (0.0, -1.0) is what I would call South, but the simulation calls this hdg=0.0, or North 
	public static final Vector2d VEC_DUE_NORTH = new Vector2d(0.0, -1.0); //due north
		
	/**
	 * Determine if targetPos has been reached by UAV at uavPos, within tolerance.
	 * For 3d vectors I am using simulations coordinates (x=E-W, y=UP-DOWN, z=N-S).
	 * TBD: should probably be using 2d vectors
	 * 
	 * @param uavPos position of the uav
	 * @param targetPos position of the target
	 * @return true if targetPos is reached within tolerance, false otherwise
	 */
	public static boolean isPosReached(Pos uavPos, Pos targetPos)
	{
		Vector3d resultant = new Vector3d(targetPos.getX() - uavPos.getX(),
		                                  targetPos.getY() - uavPos.getY(),
		                                  targetPos.getZ() - uavPos.getZ());

		double d = resultant.length();
		if (resultant.length() <= NAV_DISTANCE_TOLERANCE)
			return true;
    else
	    return false;
	}
	
	/**
	 * Return difference in height between current uavPos and targetPos.
	 * 
	 * @param uavPos position of the uav
	 * @param targetPos position of the target
	 * @return difference in height between current uavPos and targetPos
	 */
	public static double deltaHeight(Pos uavPos, Pos targetPos)
	{
		return targetPos.getY() - uavPos.getY();
	}
	
	/**
	 * Return difference in heading between current uavPos and targetPos.
	 * For 2d vectors I am using conventional coordinates: (x=E-W, y=N-S).
	 * 
	 * @param uavPos position of the uav
	 * @param targetPos position of the target
	 * @return difference in heading
	 */
	public static double deltaHeading(Pos uavPos, Pos targetPos)
	{		
		double angleUAVToTarget = 
			heading(uavPos.getZ(), uavPos.getX(), targetPos.getZ(), targetPos.getX());			
		
		double uavNeedsToGo = angleUAVToTarget - uavPos.getHdg();  
		
		if (uavNeedsToGo <= -180.0)
			uavNeedsToGo += 360;
	  else if (uavNeedsToGo > 180.0)
	  	uavNeedsToGo -= 360;
	  	
		return uavNeedsToGo;
	}
	
	/**
	 * Return difference in heading between current uavHdg and targetHdg.
	 * 
	 * @param uavHdg current heading of UAV
	 * @param targetHdg desired hdg
	 * @return difference in heading
	 */
	public static double deltaHeading(double uavHdg, double targetHdg)
	{		
		double uavNeedsToGo = targetHdg - uavHdg;
		
		if (uavNeedsToGo <= -180.0)
			uavNeedsToGo += 360;
		else if (uavNeedsToGo > 180.0)
			uavNeedsToGo -= 360;
	  	
		return uavNeedsToGo;
	}
		
	
	/**
	 * Computes heading in degrees between two locations.
	 *
	 * @param n1 first location northing
	 * @param e1 first location easting
	 * @param n2 second location northing
	 * @param e2 second location easting
	 * @return heading (degrees)
	 */
	public static double heading(double n1, double e1, double n2, double e2)
	{
		// This transformation is necessary to move from standard x,y orientation to
		// the OpenGL coordinate system used by the sim.
		///n1 = ((double)UAVMap.MAP_SIZE) - n1;
		///n2 = ((double)UAVMap.MAP_SIZE) - n2;
		
		n1 = cartesianY(n1);
		n2 = cartesianY(n2);		
		
		if (e2>e1 && n2>n1)
			return Math.toDegrees( Math.atan( (e2-e1) / (n2-n1) ) );

		else if ((e2>e1 && n2<n1) || (e2<e1 && n2<n1))
			return 180.0 + Math.toDegrees( Math.atan( (e2-e1) / (n2-n1) ) );

		else if (e2<e1 && n2>n1)
			return 360.0 + Math.toDegrees( Math.atan( (e2-e1) / (n2-n1) ) );

		else if (n2 == n1)
		{
			if (e2 > e1) return 90.0;
			else if (e2 < e1) return 270.0;
		}

		else if ((e2 == e1) && (n2 < n1))
			return 180.0;
    
		return 0.0;
	}	

	/**
	 * Return an adjusted version of Y based on rotations req'd to get simulation coordinates in line
	 * with standard cartesian coordinates.  This version uses double values for coordinates.
	 * 
	 * @param y y cartesian coordinate component to correct
	 * @return corrected y value.
	 */
	public static double cartesianY(double y)
	{
		return ((double) UAVMap.MAP_SIZE) - y;	
	}
	
	/**
	 * Return an adjusted version of Y based on rotations req'd to get simulation coordinates in line
	 * with standard cartesian coordinates.  This version uses int values for coordinates.
	 * 
	 * @param y y cartesian coordinate component to correct
	 * @return corrected y value.
	 */
	public static int cartesianY(int y)
	{
		return UAVMap.MAP_SIZE - y;
	}
	
	/**
	 * Return distance between two ordered pairs.
	 * 
	 * @param x1 ordered pair 1, x component
	 * @param y1 ordered pair 1, y component
	 * @param x2 ordered pair 2, x component
	 * @param y2 ordered pair 2, y component
	 * @return distance between the two specified ordered pairs
	 */
	public static double distance(double x1, double y1, double x2, double y2)
	{
    double dX = Math.abs(x1 - x2);
    double dY = Math.abs(y1 - y2);
    return Math.sqrt(dX+dY);
	}
	
	/**
	 * Return 2D distance between two Positions.
	 * 
	 * @param pos1
	 * @param pos2
	 * @return
	 */
	public static double distance(Pos pos1, Pos pos2)
	{
		return distance(pos1.getX(), pos1.getZ(), pos2.getX(), pos2.getZ());
	}
	
	/**
	 * Determine coordinates for UAVs in formation based on leader's position and heading.
	 * (This method is intentionally verbose for trig function clarity)
	 * 
	 * Formation:
	 * 
	 *           UAV           
	 *            |
	 *            |
	 *            |
	 *        L-------R
	 * 
	 * @param lx x coord of leader
	 * @param ly y coord of leader (== Z in sim coords)
	 * @param lhdg leader's heading in degrees (simulation heading!)
	 * @param db desired distince behind leader (and lateral distance between followers in this rank)
	 * @return double array (x1, y1, x2, y2 respectively in elements 0-3) (Y==Z in simulation coords)
	 */
	public static double[] getFormationLocation(double lx, double ly, double lhdg, double db)
	{
		double[] retArray = new double[4];
		
		if (lhdg>0.0 && lhdg<90.0)
		{
			// determine the x,y of the position behind (pb)  pbx, pby
			
			// offsets
			double a = lhdg; //angle a (degrees), basis for trig functions
			double ar = Math.toRadians(a); // angle a (radians)
			double dx = Math.sin(ar) * db;
			double dy = Math.cos(ar) * db;
			
			// absolutes
			double pbx = lx - dx;
			double pby = ly + dy;
			
			// now fan left and right from the position behind (distance between will == db)
			dx = Math.cos(ar) * (db/2);
			dy = Math.sin(ar) * (db/2);
			
			// absolutes for "left" uav
			retArray[0] = pbx - dx; //x1
			retArray[1] = pby - dy; //y1
			
			//absoutues for "right" uau
			retArray[2] = pbx + dx; //x2
			retArray[3] = pby + dy; //y2
		}

		else if (lhdg>90.0 && lhdg<180.0)
		{
			// determine the x,y of the position behind (pb)  pbx, pby
			
			// offsets
			double a = 180 - lhdg; //angle a (degrees), basis for trig functions
			double ar = Math.toRadians(a); // angle a (radians)
			double dx = Math.sin(ar) * db;
			double dy = Math.cos(ar) * db;
			
			// absolutes
			double pbx = lx - dx;
			double pby = ly - dy;
			
			// now fan left and right from the position behind (distance between will == db)
			double a1 = 90 - a;
			double ar1 = Math.toRadians(a1); 
			dx = Math.sin(ar1) * (db/2);
			dy = Math.cos(ar1) * (db/2);
			
			// absolutes for "left" uav
			retArray[0] = pbx + dx; //x1
			retArray[1] = pby - dy; //y1
			
			//absoutues for "right" uau
			retArray[2] = pbx - dx; //x2
			retArray[3] = pby + dy; //y2
		}
		
		else if (lhdg>180.0 && lhdg<270.0)
		{
			// determine the x,y of the position behind (pb)  pbx, pby
			
			// offsets
			double a = lhdg - 180; //angle a (degrees), basis for trig functions
			double ar = Math.toRadians(a); // angle a (radians)
			double dx = Math.sin(ar) * db;
			double dy = Math.cos(ar) * db;
			
			// absolutes
			double pbx = lx + dx;
			double pby = ly - dy;
			
			// now fan left and right from the position behind (distance between will == db)
			double a1 = 90 - a;
			double ar1 = Math.toRadians(a1); 
			dx = Math.sin(ar1) * (db/2);
			dy = Math.cos(ar1) * (db/2);
			
			// absolutes for "left" uav
			retArray[0] = pbx + dx; //x1
			retArray[1] = pby + dy; //y1
			
			//absoutues for "right" uau
			retArray[2] = pbx - dx; //x2
			retArray[3] = pby - dy; //y2
		}		
		
		else if (lhdg>270.0 && lhdg<360.0)
		{
			// determine the x,y of the position behind (pb)  pbx, pby
			
			// offsets
			double a = 360 - lhdg; //angle a (degrees), basis for trig functions
			double ar = Math.toRadians(a); // angle a (radians)
			double dx = Math.sin(ar) * db;
			double dy = Math.cos(ar) * db;
			
			// absolutes
			double pbx = lx + dx;
			double pby = ly + dy;
			
			// now fan left and right from the position behind (distance between will == db)
			double a1 = 90 - a;
			double ar1 = Math.toRadians(a1); 
			dx = Math.sin(ar1) * (db/2);
			dy = Math.cos(ar1) * (db/2);
			
			// absolutes for "left" uav
			retArray[0] = pbx - dx; //x1
			retArray[1] = pby + dy; //y1
			
			//absoutues for "right" uau
			retArray[2] = pbx + dx; //x2
			retArray[3] = pby - dy; //y2
		}	
				
 	  else if (lhdg==0.0 || lhdg==360.0) return new double[]{lx-(db/2), ly+db, lx+(db/2), ly+db};
		else if (lhdg==90.0) return new double[]{lx-db, ly-(db/2), lx-db, ly+(db/2)}; 
		else if (lhdg==180.0) return new double[]{lx+(db/2), ly-db, lx-(db/2), ly+db}; 
		else if (lhdg==270.0) return new double[]{lx+db, ly+(db/2), lx+db, ly-(db/2)}; 
 	  
 		return retArray;
	}
	
	/**
	 * Display array returned by getFormationLocation() in readable format.
	 * @param d double array to display.
	 */
	public static void displayFormationLocation(double[] d)
	{
		System.out.println("Left(x,y): " + d[0]+ "," + d[1] + " Right(x,y): " + d[2] + "," + d[3] + ")");
	}
	
	/**
	 * For testing
	 * @param args
	 */
  public static void main(String args[])
  {                                                           //DEGREES IN SIM:
		System.out.println("1. " + heading(68, 40, 94, 968));     //~90
		System.out.println("2. " + heading(1020, 875, 94, 968));  //~0/360
		System.out.println("3. " + heading(1010, 45, 94, 968));   //~45
 	
 	  // test getFormationLocation
 	  
 	  // 0<=hdg<=90
 	  double[] array = getFormationLocation(400, 100, 1, 10);  // ~ 395,110 ; 405,110
 	  displayFormationLocation(array);
 	  
		double[] array2 = getFormationLocation(400, 100, 45, 10); 
		displayFormationLocation(array2);
		
		// 90<=hdg<=180
		double[] array3 = getFormationLocation(400, 100, 91, 10); // ~ 390,95 ; 390,105
		displayFormationLocation(array3);		
		
		double[] array4 = getFormationLocation(400, 100, 135, 10);
		displayFormationLocation(array4);				
		
		// 180<=hdg<=270
		double[] array5 = getFormationLocation(400, 100, 181, 10); // ~ 405,90 ; 395,90
		displayFormationLocation(array5);
		
		double[] array6 = getFormationLocation(400, 100, 225, 10); 
		displayFormationLocation(array6);		
		
		// 270<=hdg<=360
		double[] array7 = getFormationLocation(400, 100, 271, 10); // ~ 410,105 ; 410,95
		displayFormationLocation(array7);
		
		double[] array8 = getFormationLocation(400, 100, 315, 10);
		displayFormationLocation(array8);				
 	  
 	  
  }
}
